// SPDX-FileCopyrightText: 2025 Fushan Wen <qydwhotmail@gmail.com>
// SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL

#pragma once

#include <KAbstractWidgetJobTracker>
#include <KDialogJobUiDelegate>
#include <KNotificationJobUiDelegate>
#include <KStatusBarJobTracker>
#include <KUiServerJobTracker>
#include <KUiServerV2JobTracker>
#include <KWidgetJobTracker>
